/*
  Copyright 2005 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <ctype.h>
#include <limits.h>
#include <gmp.h>

#include "mat.h"


mpz_t*
init_vec (int n)
{
  mpz_t *a;
  int i;

  a = (mpz_t*) malloc (n * sizeof (mpz_t));
  for (i=0; i<n; i++)
    mpz_init (a[i]);
  return a;
}

void
clear_vec (mpz_t *a, int n)
{
  int i;

  for (i=0; i<n; i++)
    mpz_clear (a[i]);
  free (a);
}

void
Print_vec (mpz_t *B, int n) 
{
  int i; 

  fprintf(stderr, "[");
  for (i=1;i<=n;i++) {
    mpz_out_str (stderr, 10, B[i]); 
    fprintf(stderr," ");
  }
  fprintf(stderr, "]\n"); 
}

void
init_matrix (mat_ZZ *B, int d, int n)
{
  int i;

  B->NumCols=d;
  B->NumRows=n; 
  B->coeff = (mpz_t**) malloc ((d + 1) * sizeof (mpz_t*));
  for (i=0; i<=d; i++)
    B->coeff[i] = init_vec (n + 1);
}


void
clear_matrix (mat_ZZ *B, int d, int n)
{
  int i;
  for (i=0; i<=d; i++)
    clear_vec (B->coeff[i], n + 1);
  free (B->coeff);
}

void
Print_mat (mpz_t **B, int d, int n) 
{
  int i, j; 

  printf("[");
  for (i=1;i<=d;i++) {
    printf("[");
    for (j=1;j<=n;j++) { 
      mpz_out_str(stdout, 10, B[i][j]); 
      if (j < n) printf(" "); 
      //mpz_out_str(stdout, 10, B[i][j]); 
      //if ((i<d) ||(j<n))   printf(", "); 
    }
    printf("]\n"); 
  }  
  printf("]\n"); 
}



/* ScalarProduct of two vectors of length n */
void 
ScalarProduct (mpz_t s, mpz_t *vec1, mpz_t *vec2, int n)
{
  int i;
  mpz_t tmp;
  mpz_init (tmp);

  mpz_mul (s, vec1[1], vec2[1]);
  for (i=2; i<=n; i++)
    {
      mpz_mul (tmp, vec1[i], vec2[i]);
      mpz_add (s, s, tmp);      
    }
  
  mpz_clear (tmp);
} 

/* Computes the Gram matrix G of the integral basis B */

void 
Gram (mat_ZZ *G, mat_ZZ *B)
{
  int i,j;
  mpz_t tmp;
  int n = B->NumRows;
  int d = B->NumCols;

  mpz_init (tmp);

  G->NumRows = G->NumCols = d;
  for(i=1; i<=d; i++)
    {
      for(j=1; j<=i; j++)
	{
	  ScalarProduct(tmp, B->coeff[i], B->coeff[j], n);
	  mpz_set(G->coeff[i][j], tmp); 
	}
    }

  mpz_clear (tmp);
}
